@@ -3,12 +3,15 @@
 
 import json
 import keyword
-import os
 import sys
+import subprocess
+import shutil
+import textwrap
 import time
 import webbrowser
 
 import click
+import numpy as np
 import rich
 from rich import prompt
 from rich.console import Console
@@ -27,6 +30,8 @@ class AliasedGroup(click.Group):
         # Add command aliases
         if cmd_name == "identify":
             cmd_name = "id"
+        elif cmd_name == "aliases":
+            cmd_name = "ids"
         elif cmd_name == "parameter":
             cmd_name = "parameters"
         elif cmd_name == "update":
@@ -57,9 +62,15 @@ def docs():
 
 
 @cli_rocks.command()
-@click.argument("id_")
+@click.argument("id_", nargs=-1)
 def id(id_):
     """Resolve the asteroid name and number from string input."""
+
+    if not id_:
+        id_ = rocks.resolve._interactive()
+    else:
+        id_ = id_[0]
+
     name, number = rocks.identify(id_)  # type: ignore
 
     if isinstance(name, (str)):
@@ -69,9 +80,15 @@ def id(id_):
 
 
 @cli_rocks.command()
-@click.argument("id_")
+@click.argument("id_", nargs=-1)
 def info(id_):
     """Print the ssoCard of an asteroid."""
+
+    if not id_:
+        id_ = rocks.resolve._interactive()
+    else:
+        id_ = id_[0]
+
     _, _, id_ = rocks.identify(id_, return_id=True)  # type: ignore
 
     if not isinstance(id_, str):
@@ -86,7 +103,7 @@ def parameters():
     """Print the ssoCard structure and its description."""
 
     if not rocks.PATH_MAPPING.is_file():
-        rocks.utils.retrieve_mappings_from_ssodnet()
+        rocks.utils.retrieve_mappings()
 
     with open(rocks.PATH_MAPPING, "r") as file_:
         DESC = json.load(file_)
@@ -94,13 +111,21 @@ def parameters():
     rich.print(DESC)
 
 
-@cli_rocks.command()
-@click.argument("id_")
-def aliases(id_):
+@cli_rocks.command(hidden=True)
+@click.argument("id_", nargs=-1)
+def ids(id_):
     """Echo the aliases of an asteroid."""
 
+    if not id_:
+        id_ = rocks.resolve._interactive()
+    else:
+        id_ = id_[0]
+
     name, number, ssodnetid = rocks.identify(id_, return_id=True)  # type: ignore
 
+    if name is None:
+        sys.exit()
+
     aliases = rocks.index.get_aliases(ssodnetid)
 
     rich.print(f"({number}) {name}, aka \n {aliases}")
@@ -114,10 +139,10 @@ def status():
     # Echo inventory
 
     # Get set of ssoCards and datacloud catalogues in cache
-    cached_cards, cached_catalogues, cached_meta = rocks.utils.cache_inventory()
+    cached_cards, cached_catalogues = rocks.utils.cache_inventory()
 
     # Get the modification date of the index
-    date_index = os.path.getmtime(rocks.PATH_INDEX / "1.pkl")
+    date_index = (rocks.PATH_INDEX / "1.pkl").stat().st_mtime
     date_index = time.strftime("%d %b %Y", time.localtime(date_index))
 
     # Print the findings
@@ -126,8 +151,7 @@ def status():
 
         {len(cached_cards)} ssoCards
         {len(cached_catalogues)} datacloud catalogues\n
-        Asteroid name-number index [blue]\[index.pkl][/blue] updated on {date_index}
-        Metadata files [blue]{cached_meta}[/blue]\n"""
+        Asteroid name-number index updated on {date_index}\n"""
     )
 
     # ------
@@ -139,10 +163,10 @@ def status():
     ):
         rich.print(
             f"[red]The running [green]rocks[/green] version ({rocks.__version__}) is behind the "
-            f"latest version ({latest_rocks}). The ssoCard structure might have changed.[/red]"
+            f"latest version ({latest_rocks}). The ssoCard structure might have changed.[/red]\n"
         )
         rich.print(
-            f"You should run [green]$ pip install -U space-rocks[/green] and clear the cache directory.\n"
+            "You should run [green]$ pip install -U space-rocks[/green] and clear the cache directory.\n"
         )
 
     # Update or clear
@@ -165,7 +189,7 @@ def status():
         elif decision == "2":
 
             # Update metadata
-            rocks.utils.retrieve_mappings_from_ssodnet()
+            rocks.utils.retrieve_mappings()
 
             # Update the cached data
             ids = [ssodnet_id for ssodnet_id in cached_cards]
@@ -199,9 +223,48 @@ def status():
     )
 
     if response == "1":
+        click.echo()
         rocks.index._build_index()
 
 
+@cli_rocks.command()
+@click.argument("id_", nargs=-1)
+def who(id_):
+    """Get name citation of asteroid from MPC."""
+
+    if not id_:
+        id_ = rocks.resolve._interactive()
+    else:
+        id_ = id_[0]
+
+    name, number = rocks.identify(id_)
+
+    if name is None:
+        sys.exit()
+
+    if not np.isnan(number):
+        id_ = number
+    else:
+        id_ = name
+
+    citation = rocks.utils.get_citation_from_mpc(id_)
+
+    if citation is None:
+        rich.print(f"({number}) {name} has no citation attached to its name.")
+        sys.exit()
+
+    # Pretty-print citation
+    rich.print(f"({number}) {name}")
+    rich.print(
+        textwrap.fill(
+            citation,
+            width=shutil.get_terminal_size()[0] - 2,
+            initial_indent="  ",
+            subsequent_indent="  ",
+        ),
+    )
+
+
 def echo():
     """Echos asteroid parameter to command line. Optionally opens plot."""
 
@@ -266,12 +329,26 @@ def echo():
         if param in datacloud:
             rocks.datacloud.pretty_print(rock, rocks.utils.rgetattr(rock, param), param)
         else:
-            print(rocks.utils.rgetattr(rock, param))
+            value = rocks.utils.rgetattr(rock, param)
+
+            if isinstance(value, rocks.core.SpinList):
+                if verbose:
+                    for entry in value:
+                        rich.print_json(entry.json(), sort_keys=True)
+                else:
+                    for entry in value:
+                        rich.print(entry)
+
+            else:
+                if verbose:
+                    rich.print_json(value.json(), sort_keys=True)
+                else:
+                    print(value)
 
         if plot:
             if param not in datacloud:
                 print(
-                    f"Only datacloud collections can be plotted. "
+                    "Only datacloud collections can be plotted. "
                     f"Try the plural of {param}."
                 )
                 sys.exit()
@@ -280,3 +357,42 @@ def echo():
 
     # Avoid error message from click
     sys.exit()
+
+
+def _interactive(LINES):
+    """Launch interactive selection using fzf."""
+
+    PATH_EXECUTABLE = shutil.which("fzf")
+
+    if PATH_EXECUTABLE is None:
+        rich.print(
+            "Interactive selection is not possible as the fzf tool is not installed.\n"
+            "Either install fzf or specify an asteroid identifier.\n"
+            "Refer to https://rocks.readthedocs.io/en/latest/getting_started.html#optional-interactive-search"
+        )
+        sys.exit()
+
+    FZF_OPTIONS = []
+
+    # Open fzf subprocess
+    process = subprocess.Popen(
+        [shutil.which("fzf"), *FZF_OPTIONS],
+        stdin=subprocess.PIPE,
+        stdout=subprocess.PIPE,
+        stderr=None,
+    )
+
+    for line in LINES:
+        process.stdin.write(line)
+        process.stdin.flush()
+
+    # Run process and wait for user selection
+    process.stdin.close()
+    process.wait()
+
+    # Extract selected line
+    try:
+        choice = [line for line in process.stdout][0].decode()
+    except IndexError:  # no choice was made, c-c c-c
+        sys.exit()
+    return choice
