@@ -1,8 +1,7 @@
 #!/usr/bin/env python
 """Implement the Rock class and other core rocks functionality."""
 import datetime as dt
-import json
-from typing import Dict, List, Optional, Union
+from typing import List
 import warnings
 
 import numpy as np
@@ -15,6 +14,48 @@ import rocks
 
 # ------
 # ssoCard as pydantic model
+ALIASES = {
+    "dynamical": {
+        "parameters.dynamical.orbital_elements": "orbital_elements",
+        "parameters.dynamical.family": "family",
+        "parameters.dynamical.pair": "pair",
+        "parameters.dynamical.proper_elements": "proper_elements",
+        "parameters.dynamical.tisserand_parameter": "tisserand_parameter",
+        "parameters.dynamical.yarkovsky": "yarkovsky",
+    },
+    "physical": {
+        "D": "diameter",
+        "H": "absolute_magnitude",
+        "parameters.physical.absolute_magnitude": "absolute_magnitude",
+        "parameters.physical.albedo": "albedo",
+        "parameters.physical.colors": "color",
+        "parameters.physical.diameter": "diameter",
+        "parameters.physical.density": "density",
+        "parameters.physical.mass": "mass",
+        "parameters.physical.phase_function": "phase_function",
+        "parameters.physical.spin": "spin",
+        "parameters.physical.taxonomy": "taxonomy",
+        "parameters.physical.thermal_inertia": "thermal_inertia",
+    },
+    "orbital_elements": {
+        "a": "semi_major_axis",
+        "e": "eccentricity",
+        "i": "inclination",
+        "P": "orbital_period",
+    },
+    "proper_elements": {
+        "ap": "proper_semi_major_axis",
+        "ep": "proper_eccentricity",
+        "ip": "proper_inclination",
+        "sinip": "proper_sine_inclination",
+    },
+    "diamalbedo": ["albedos", "diameters"],
+    "phase_function": {
+        "V": "generic_johnson_V",
+        "cyan": "misc_atlas_cyan",
+        "orange": "misc_atlas_orange",
+    },
+}
 
 # The lowest level in the ssoCard tree is the Value
 class Error(pydantic.BaseModel):
@@ -123,8 +164,8 @@ class Bibref(Parameter):
 
 
 class LinksParameter(Parameter):
-    datacloud: StringValue = StringValue(**{})
-    selection: StringValue = StringValue(**{})
+    datacloud: str = ""
+    selection: str = ""
 
 
 # And a special class for the Spin list
@@ -133,48 +174,19 @@ class SpinList(list):
 
     def __init__(self, list_):
         """Convert the list items to Spin instances."""
-        list_ = [Spin(**entry) for entry in list_]
+
+        if list_:  # spin list is populated
+            list_ = [Spin(**entry) for entry in list_]
+        else:
+            list_ = [Spin(**{})]  # ensure it's never empty
+
         return super().__init__(list_)
 
     def __str__(self) -> str:
         return "\n".join(entry.json() for entry in self)
 
-
-# ------
-# Validators
-# def convert_spin_to_list(spins: Dict) -> List:
-#     """Convert the Spin dictionary from the ssoCard into a list.
-#     Add the spin index as parameter to the Spin entries.
-
-#     Parameters
-#     ----------
-#     spin : dict
-#         The dictionary located at parameters.physical.spin in the ssoCard.
-
-#     Returns
-#     -------
-#     list
-#         A list of dictionaries, with one dictionary for each entry in parameters.physical.spin
-#         after removing the index layer.
-#     """
-#     spin_solutions = []
-
-#     for key, spin in spins.items():
-
-#         # spin entries have integer ids
-#         if not key.isnumeric():
-#             continue
-
-#         # convert the spin key to an entry in the solution dict
-#         spin["id_"] = {
-#             "value": key,
-#             "description": "Identifier of the spin axis solution",
-#             "format": "%s",
-#         }
-
-#         spin_solutions.append(spin)
-
-#     return spin_solutions
+    def __bool__(self) -> bool:
+        return any(np.isfinite(entry.period.value) for entry in self)
 
 
 # ------
@@ -199,8 +211,6 @@ class OrbitalElements(Parameter):
     number_observation: FloatValue = FloatValue(**{})
     perihelion_argument: FloatValue = FloatValue(**{})
 
-    # _add_metadata: classmethod = pydantic.validator("semi_major_axis")(_add_metadata)
-
 
 class ProperElements(Parameter):
     links: LinksParameter = LinksParameter(**{})
@@ -232,7 +242,7 @@ class Family(Parameter):
 
 
 class Pair(Parameter):
-    age: Value = Value(**{})
+    age: FloatValue = FloatValue(**{})
     distance: FloatValue = FloatValue(**{})
     sibling_name: StringValue = StringValue(**{})
     sibling_number: IntegerValue = IntegerValue(**{})
@@ -245,10 +255,10 @@ class TisserandParameter(Parameter):
 
 
 class Yarkovsky(Parameter):
-    S: float = np.nan
-    A2: Value = Value(**{})
+    S: FloatValue = FloatValue(**{})
+    A2: FloatValue = FloatValue(**{})
     snr: FloatValue = FloatValue(**{})
-    dadt: Value = Value(**{})
+    dadt: FloatValue = FloatValue(**{})
     bibref: List[Bibref] = [Bibref(**{})]
     method: List[Method] = [Method(**{})]
 
@@ -257,10 +267,10 @@ class Yarkovsky(Parameter):
 
 
 class DynamicalParameters(Parameter):
-    # pair: Pair = pydantic.Field(Pair(**{}), alias="pairs")
-    # family: Family = Family(**{})
+    pair: Pair = pydantic.Field(Pair(**{}), alias="pairs")
+    family: Family = Family(**{})
     tisserand_parameter: TisserandParameter = TisserandParameter(**{})
-    # yarkovsky: Yarkovsky = Yarkovsky(**{})
+    yarkovsky: Yarkovsky = Yarkovsky(**{})
     proper_elements: ProperElements = ProperElements(**{})
     orbital_elements: OrbitalElements = OrbitalElements(**{})
 
@@ -277,7 +287,7 @@ class Albedo(FloatValue):
 
 
 class ColorEntry(Parameter):
-    color: Value = Value(**{})
+    color: FloatValue = FloatValue(**{})
     epoch: FloatValue = FloatValue(**{})
     links: LinksParameter = LinksParameter(**{})
     method: List[Method] = []
@@ -337,10 +347,10 @@ class Mass(FloatValue):
 
 
 class Phase(Parameter):
-    H: Value = Value(**{})
+    H: FloatValue = FloatValue(**{})
     N: FloatValue = FloatValue(**{})
-    G1: Value = Value(**{})
-    G2: Value = Value(**{})
+    G1: FloatValue = FloatValue(**{})
+    G2: FloatValue = FloatValue(**{})
     rms: FloatValue = FloatValue(**{})
     phase: Error = Error(**{})
     links: LinksParameter = LinksParameter(**{})
@@ -348,25 +358,64 @@ class Phase(Parameter):
     facility: StringValue = StringValue(**{})
     name_filter: StringValue = StringValue(**{})
 
+    def __bool__(self):
+        return bool(np.isfinite(self.H.value))
+
+    def __str__(self):
+        if not np.isnan(self.H.value):
+            return rf"H: {self.H.value:.2f}  G1: {self.G1.value:.2f}  G2: {self.G2.value:.2f}  [{self.bibref[0].shortbib}]"
+        return "No phase function on record in this filter."
+
 
 class PhaseFunction(Parameter):
     # Generic
-    generic_johnson_v: Phase = pydantic.Field(Phase(**{}), alias="Generic/Johnson.V")
+    generic_johnson_V: Phase = pydantic.Field(Phase(**{}), alias="Generic/Johnson.V")
     # ATLAS
     misc_atlas_cyan: Phase = pydantic.Field(Phase(**{}), alias="Misc/Atlas.cyan")
     misc_atlas_orange: Phase = pydantic.Field(Phase(**{}), alias="Misc/Atlas.orange")
 
+    def __getattr__(self, name):
+        """Implement attribute shortcuts. Gets called if __getattribute__ fails."""
+
+        if name in ALIASES["phase_function"].keys():
+            return getattr(self, ALIASES["phase_function"][name])
+
+    def __bool__(self):
+        return any(
+            [
+                np.isfinite(getattr(self, filter_).H.value)
+                for filter_ in [
+                    "generic_johnson_V",
+                    "misc_atlas_cyan",
+                    "misc_atlas_orange",
+                ]
+            ]
+        )
+
+    def __repr__(self):
+        observed = []
+
+        for filter_ in ["generic_johnson_V", "misc_atlas_cyan", "misc_atlas_orange"]:
+            entry = getattr(self, filter_)
+            if not np.isnan(entry.H.value):
+                observed.append(
+                    rf"H: {entry.H.value:.2f}  G1: {entry.G1.value:.2f}  G2: {entry.G2.value:.2f}  \[{filter_}]"
+                )
+        if observed:
+            return "\n".join(observed)
+        return "No phase function on record."
+
 
 class Spin(Parameter):
     t0: FloatValue = FloatValue(**{})
     Wp: FloatValue = FloatValue(**{})
     id_: IntegerValue = IntegerValue(**{})
-    lat: Value = Value(**{})
-    RA0: Value = Value(**{})
-    DEC0: Value = Value(**{})
+    lat: FloatValue = FloatValue(**{})
+    RA0: FloatValue = FloatValue(**{})
+    DEC0: FloatValue = FloatValue(**{})
     links: LinksParameter = LinksParameter(**{})
-    long_: Value = pydantic.Field(Value(**{}), alias="long")
-    period: Value = Value(**{})
+    long_: FloatValue = pydantic.Field(FloatValue(**{}), alias="long")
+    period: FloatValue = FloatValue(**{})
     obliquity: FloatValue = FloatValue(**{})
     method: List[Method] = [Method(**{})]
     bibref: List[Bibref] = [Bibref(**{})]
@@ -383,10 +432,13 @@ class Taxonomy(Parameter):
     technique: StringValue = StringValue(**{})
     waverange: StringValue = StringValue(**{})
 
+    def __bool__(self):
+        return bool(self.class_.value)
+
     def __str__(self):
         if not self.class_:
             return "No taxonomy on record."
-        return self.class_
+        return self.class_.value
 
 
 class ThermalInertia(FloatValue):
@@ -410,14 +462,12 @@ class PhysicalParameters(Parameter):
     density: Density = Density(**{})
     diameter: Diameter = Diameter(**{})
     taxonomy: Taxonomy = Taxonomy(**{})
-    phase_function: PhaseFunction = PhaseFunction(**{})
+    phase_function: PhaseFunction = pydantic.Field(
+        PhaseFunction(**{}), alias="phase_functions"
+    )
     thermal_inertia: ThermalInertia = ThermalInertia(**{})
     absolute_magnitude: AbsoluteMagnitude = AbsoluteMagnitude(**{})
 
-    # Can probably reduce verbosity here
-    # _convert_spin_to_list: classmethod = pydantic.validator("spin", pre=True)(
-    #     convert_spin_to_list
-    # )
     _convert_list_to_parameterlist: classmethod = pydantic.validator(
         "spin", allow_reuse=True, pre=True
     )(lambda list_: SpinList(list_))
@@ -474,7 +524,7 @@ class Rock(pydantic.BaseModel):
 
     # the catalogues
     astorb: rocks.datacloud.Astorb = rocks.datacloud.Astorb(**{})
-    binarymp: rocks.datacloud.Binarymp = rocks.datacloud.Binarymp(**{})
+    binaries: rocks.datacloud.Binarymp = rocks.datacloud.Binarymp(**{})
     colors: rocks.datacloud.Colors = rocks.datacloud.Colors(**{})
     densities: rocks.datacloud.Density = rocks.datacloud.Density(**{})
     diamalbedo: rocks.datacloud.Diamalbedo = rocks.datacloud.Diamalbedo(**{})
@@ -486,7 +536,7 @@ class Rock(pydantic.BaseModel):
     proper_elements: rocks.datacloud.Proper_elements = rocks.datacloud.Proper_elements(
         **{}
     )
-    phase_function: rocks.datacloud.Phase_function = rocks.datacloud.Phase_function(
+    phase_functions: rocks.datacloud.Phase_function = rocks.datacloud.Phase_function(
         **{}
     )
     taxonomies: rocks.datacloud.Taxonomies = rocks.datacloud.Taxonomies(**{})
@@ -670,33 +720,33 @@ class Rock(pydantic.BaseModel):
         """Implement attribute shortcuts. Gets called if __getattribute__ fails."""
 
         # These are shortcuts
-        if name in self.__aliases["physical"].values():
+        if name in ALIASES["physical"].values():
             return getattr(self.parameters.physical, name)
 
-        if name in self.__aliases["dynamical"].values():
+        if name in ALIASES["dynamical"].values():
             return getattr(self.parameters.dynamical, name)
 
         # TODO This could be coded in a more abstract way
         # These are proper aliases
-        if name in self.__aliases["orbital_elements"].keys():
+        if name in ALIASES["orbital_elements"].keys():
             return getattr(
                 self.parameters.dynamical.orbital_elements,
-                self.__aliases["orbital_elements"][name],
+                ALIASES["orbital_elements"][name],
             )
 
-        if name in self.__aliases["proper_elements"].keys():
+        if name in ALIASES["proper_elements"].keys():
             return getattr(
                 self.parameters.dynamical.proper_elements,
-                self.__aliases["proper_elements"][name],
+                ALIASES["proper_elements"][name],
             )
 
-        if name in self.__aliases["physical"].keys():
+        if name in ALIASES["physical"].keys():
             return getattr(
                 self.parameters.physical,
-                self.__aliases["physical"][name],
+                ALIASES["physical"][name],
             )
 
-        if name in self.__aliases["diamalbedo"]:
+        if name in ALIASES["diamalbedo"]:
             return getattr(self, "diamalbedo")
 
         raise AttributeError(
@@ -739,13 +789,13 @@ class Rock(pydantic.BaseModel):
         # turn list of dict (catalogue entries) into dict of list
         cat = {
             key: [c[key] if key in c else "" for c in cat]
-            if catalogue not in ["aams", "astorb", "families"]
+            if catalogue not in ["aams", "families"]
             else cat[0][key]
             for key in cat[0].keys()
         }
 
         # add 'preferred' attribute where applicable
-        if catalogue_ssodnet in ["taxonomy", "masses", "diamalbedo"]:
+        if catalogue_ssodnet in ["thermal_inertia", "taxonomy", "masses", "diamalbedo"]:
             cat["preferred"] = [False] * len(list(cat.values())[0])
         if catalogue_ssodnet in ["diamalbedo"]:
             cat["preferred_albedo"] = [False] * len(list(cat.values())[0])
@@ -779,43 +829,6 @@ class Rock(pydantic.BaseModel):
                 f"\nPassed value: {value}"
             )
 
-    __aliases = {
-        "dynamical": {
-            "parameters.dynamical.orbital_elements": "orbital_elements",
-            "parameters.dynamical.family": "family",
-            "parameters.dynamical.pair": "pair",
-            "parameters.dynamical.proper_elements": "proper_elements",
-            "parameters.dynamical.tisserand_parameter": "tisserand_parameter",
-            "parameters.dynamical.yarkovsky": "yarkovsky",
-        },
-        "physical": {
-            "D": "diameter",
-            "H": "absolute_magnitude",
-            "parameters.physical.absolute_magnitude": "absolute_magnitude",
-            "parameters.physical.albedo": "albedo",
-            "parameters.physical.colors": "color",
-            "parameters.physical.diameter": "diameter",
-            "parameters.physical.density": "density",
-            "parameters.physical.mass": "mass",
-            "parameters.physical.phase_function": "phase_function",
-            "parameters.physical.spin": "spin",
-            "parameters.physical.taxonomy": "taxonomy",
-            "parameters.physical.thermal_inertia": "thermal_inertia",
-        },
-        "orbital_elements": {
-            "a": "semi_major_axis",
-            "e": "eccentricity",
-            "i": "inclination",
-        },
-        "proper_elements": {
-            "ap": "proper_semi_major_axis",
-            "ep": "proper_eccentricity",
-            "ip": "proper_inclination",
-            "sinip": "proper_sine_inclination",
-        },
-        "diamalbedo": ["albedos", "diameters"],
-    }
-
 
 def rocks_(ids, datacloud=None, progress=False, suppress_errors=False):
     """Create multiple Rock instances.
