@@ -1,6 +1,7 @@
 #!/usr/bin/env python
 """Implement the Datacloud catalogue pydantic models."""
 
+import re
 from typing import List, Optional
 
 import numpy as np
@@ -30,7 +31,8 @@ CATALOGUES = {
             "err_diameter_up",
             "err_diameter_down",
             "method",
-            "bibcode",
+            # "bibcode",
+            "shortbib",
         ],
     },
     "astdys": {
@@ -54,17 +56,24 @@ CATALOGUES = {
             "H",
             "G",
             "B_V",
-            "IRAS_Diameter",
-            "IRAS_Class",
-            "SemimajorAxis",
-            "Eccentricity",
-            "Inclination",
+            "IRAS_diameter",
+            "IRAS_class",
+            "semi_major_axis",
+            "eccentricity",
+            "inclination",
         ],
     },
     "binarymp": {
         "attr_name": "binaries",
         "ssodnet_name": "binarymp",
-        "print_columns": [""],
+        "print_columns": [
+            "system_type",
+            "system_name",
+            "period",
+            "a",
+            "alpha",
+            "shortbib",
+        ],
     },
     "colors": {
         "attr_name": "colors",
@@ -131,7 +140,14 @@ CATALOGUES = {
     "mpcorb": {
         "attr_name": "mpcorb",
         "ssodnet_name": "mpcorb",
-        "print_columns": [],
+        "print_columns": [
+            "H",
+            "G",
+            "semi_major_axis",
+            "eccentricity",
+            "inclination",
+            "orbital_arc",
+        ],
     },
     "pairs": {
         "attr_name": "pairs",
@@ -141,7 +157,15 @@ CATALOGUES = {
     "phase_functions": {
         "attr_name": "phase_functions",
         "ssodnet_name": "phase_function",
-        "print_columns": [],
+        "print_columns": [
+            "name_filter",
+            "H",
+            "G1",
+            "G2",
+            "phase_min",
+            "phase_max",
+            "shortbib",
+        ],
     },
     "spins": {
         "attr_name": "spins",
@@ -172,10 +196,19 @@ CATALOGUES = {
             "shortbib",
         ],
     },
-    "yarkovskies": {
-        "attr_name": "yarkovskies",
+    "yarkovskys": {
+        "attr_name": "yarkovskys",
         "ssodnet_name": "yarkovsky",
-        "print_columns": [],
+        "print_columns": [
+            "A2",
+            "err_A2",
+            "dadt",
+            "err_dadt",
+            "S",
+            "snr",
+            "method",
+            "shortbib",
+        ],
     },
 }
 
@@ -197,27 +230,27 @@ def pretty_print(rock, catalogue, parameter):
         print(f"No {parameter} on record for ({rock.number}) {rock.name}.")
         return
 
+    # Sort catalogue by year of reference
+    if "year" in catalogue.columns:
+        catalogue = catalogue.sort_values("year").reset_index()
+
     # ------
     # Create table to echo
     if parameter in ["diameters", "albedos"]:
-
         if parameter == "diameters":
             preferred = catalogue.preferred_diameter
-            caption = "Green: preferred entry"
         elif parameter == "albedos":
             preferred = catalogue.preferred_albedo
-            caption = "Green: preferred entry"
-        # else:
-        #     preferred = catalogue.preferred
-        #     caption = "Blue: preferred diameter, yellow: preferred albedo, green: both preferred"
-
-    elif parameter in ["masses", "taxonomies", "diamalbedo"]:
-        caption = "Green: preferred entry"
+    elif hasattr(catalogue, "preferred"):
         preferred = catalogue.preferred
+    else:
+        preferred = [False for _ in range(len(catalogue))]
 
+    # Only show the caption if there is a preferred entry
+    if any(preferred):
+        caption = "Green: preferred entry"
     else:
         caption = None
-        preferred = [False for _ in range(len(catalogue))]
 
     table = Table(
         header_style="bold blue",
@@ -227,10 +260,6 @@ def pretty_print(rock, catalogue, parameter):
         caption=caption,
     )
 
-    # Sort catalogue by year of reference
-    if "year" in catalogue.columns:
-        catalogue = catalogue.sort_values("year").reset_index()
-
     # The columns depend on the catalogue
     columns = [""] + CATALOGUES[parameter]["print_columns"]
 
@@ -338,9 +367,51 @@ def empty_str_to_nan(value):
     return value
 
 
+def get_preferred(name, parameter, ids):
+    """Get the preferred values for this catalogue from the ssoCard of the object.
+
+    Parameters
+    ----------
+    name : str
+        The asteroid name, extracted from the datacloud catalogue.
+    parameter : str
+        The full parameter path in the ssoCard.
+    ids : list of int
+        List of dataset ids present in the datacloud catalogue.
+
+    Returns
+    -------
+    list of bool
+        True if the id belongs to a preferred entry, else False.
+    """
+
+    # Get ssoCard
+    ssoCard = rocks.Rock(name)
+
+    # Get selected parameters
+    link_selection = rocks.utils.rgetattr(ssoCard, f"{parameter}.links.selection")
+
+    if not link_selection:
+        return [False for id_ in ids]
+
+    # Parse selection link for preferred dataset ids
+    match = re.search(r"(.+)(:id=)([0-9,]+)(.*)", link_selection)
+    ids_preferred = match.group(3)
+    ids_preferred = [int(id_) for id_ in ids_preferred.split(",")]
+
+    # Create list of preferred dataset ids
+    preferred = [True if id_ in ids_preferred else False for id_ in ids]
+
+    # If all are preferred, none are preferred
+    if all(preferred):
+        preferred = [False for _ in preferred]
+
+    return preferred
+
+
 # ------
 # SsODNet catalogues as pydantic model
-# https://ssp.imcce.fr/webservices/ssodnet/api/datacloud/templates/ssodnet_datacloud.sql
+# https://ssp.imcce.fr/data/ssodnet_datacloud.sql
 
 # Metacatalogues
 class Collection(pydantic.BaseModel):
@@ -352,6 +423,9 @@ class Collection(pydantic.BaseModel):
     datasetname: List[str] = [""]
     idcollection: List[int] = [None]
     resourcename: List[str] = [""]
+    bibcode: List[str] = [""]
+    doi: List[str] = [""]
+    year: List[int] = [None]
 
 
 class Methods(pydantic.BaseModel):
@@ -638,6 +712,7 @@ class Binarymp(pydantic.BaseModel):
 class Diamalbedo(Collection):
     """Diameters and Albedos database from literature"""
 
+    id_: List[int] = pydantic.Field([None], alias="id")
     number: List[Optional[int]] = pydantic.Field([None], alias="num")
     name: List[str] = [""]
     diameter: List[float] = [np.nan]
@@ -658,13 +733,17 @@ class Diamalbedo(Collection):
     preferred_diameter: List[bool] = [False]
     preferred: List[bool] = [False]
 
-    @pydantic.validator("preferred_albedo", pre=True)
-    def select_preferred_albedo(cls, _, values):
-        return rocks.definitions.rank_properties("albedo", values)
+    @pydantic.validator("preferred_albedo", pre=True, allow_reuse=True)
+    def select_preferred(cls, _, values):
+        return get_preferred(
+            values["name"][0], "parameters.physical.albedo", values["id_"]
+        )
 
-    @pydantic.validator("preferred_diameter", pre=True)
-    def select_preferred_diameter(cls, _, values):
-        return rocks.definitions.rank_properties("diameter", values)
+    @pydantic.validator("preferred_diameter", pre=True, allow_reuse=True)
+    def select_preferred(cls, _, values):
+        return get_preferred(
+            values["name"][0], "parameters.physical.diameter", values["id_"]
+        )
 
     @pydantic.validator("preferred", pre=True)
     def preferred_albedo_or_diameter(cls, _, values):
@@ -692,14 +771,16 @@ class Masses(Collection):
     preferred: List[bool] = [False]
 
     @pydantic.validator("preferred", pre=True)
-    def select_preferred_mass(cls, _, values):
-        return rocks.definitions.rank_properties("mass", values)
+    def select_preferred(cls, _, values):
+        return get_preferred(
+            values["name"][0], "parameters.physical.mass", values["id_"]
+        )
 
 
 class Taxonomies(Collection):
     """Taxonomy database from literature"""
 
-    # id_: List[int] = pydantic.Field([None], alias="id")
+    id_: List[int] = pydantic.Field([None], alias="id")
     number: List[Optional[int]] = pydantic.Field([None], alias="num")
     name: List[str] = [""]
     year: List[int] = [None]
@@ -715,8 +796,10 @@ class Taxonomies(Collection):
     preferred: List[bool] = [False]
 
     @pydantic.validator("preferred", pre=True)
-    def select_preferred_class(cls, _, values):
-        return rocks.definitions.rank_properties("taxonomy", values)
+    def select_preferred(cls, _, values):
+        return get_preferred(
+            values["name"][0], "parameters.physical.taxonomy", values["id_"]
+        )
 
 
 class Proper_elements(Collection):
@@ -880,6 +963,14 @@ class Thermal_inertia(Collection):
     method: List[str] = [""]
     iddataset: List[int] = [None]
 
+    preferred: List[bool] = [False]
+
+    @pydantic.validator("preferred", pre=True)
+    def select_preferred(cls, _, values):
+        return get_preferred(
+            values["name"][0], "parameters.physical.thermal_inertia", values["id_"]
+        )
+
 
 class Colors(Collection):
     """Database of asteroid colors"""
