@@ -1,7 +1,6 @@
 #!/usr/bin/env python
+"""Implement SsODNet:Datacloud queries."""
 
-"""Implement SsODNet:Datacloud queries.
-"""
 import asyncio
 from itertools import product
 import warnings
@@ -151,7 +150,17 @@ def _postprocess_ssocard(card):
                 make_dict(value)
             else:
                 # These keys are not touched, they don't have metadata
-                if key in ["bibref", "method", "value", "error", "min", "max"]:
+                if key in [
+                    "bibref",
+                    "method",
+                    "value",
+                    "error",
+                    "min",
+                    "max",
+                    "links",
+                    "datacloud",
+                    "selection",
+                ]:
                     continue
                 # Turn non-dict value into dict for merging with metadata
                 values[key] = {"value": value}
@@ -161,15 +170,19 @@ def _postprocess_ssocard(card):
     card["parameters"] = make_dict(card["parameters"])
 
     # Catch for spins
-    for id_ in card["parameters"]["physical"]["spins"]:
-        mappings["parameters"]["physical"]["spins"][id_] = mappings["parameters"][
-            "physical"
-        ]["spins"]["<id>"]
+    if "spins" in card["parameters"]["physical"]:
+        for id_ in card["parameters"]["physical"]["spins"]:
+            mappings["parameters"]["physical"]["spins"][id_] = mappings["parameters"][
+                "physical"
+            ]["spins"]["<id>"]
 
     card = deep_update(card, mappings)
 
     # ------
     # Convert spin to list
+    if "spins" not in card["parameters"]["physical"]:
+        card["parameters"]["physical"]["spins"] = {}
+
     spins = card["parameters"]["physical"]["spins"]
     spin_solutions = []
 
@@ -182,8 +195,6 @@ def _postprocess_ssocard(card):
         # convert the spin key to an entry in the solution dict
         spin["id_"] = {
             "value": key,
-            "description": "Identifier of the spin axis solution",
-            "format": "%s",
         }
 
         spin_solutions.append(spin)
