@@ -56,11 +56,7 @@ def get_unit(path_parameter: str) -> str:
     str
         The unit of the requested parameter.
     """
-    if not rocks.PATH_MAPPING.is_file():
-        retrieve_mappings_from_ssodnet()
-
-    with open(rocks.PATH_MAPPING, "r") as file_:
-        mappings = json.load(file_)
+    mappings = load_mappings()
 
     if "unit" in mappings[path_parameter]:
         unit = mappings[path_parameter]["unit"]
@@ -70,6 +66,17 @@ def get_unit(path_parameter: str) -> str:
     return unit
 
 
+def load_mappings():
+    """Load SsODNet metadata mappings file from cache."""
+    if not rocks.MAPPINGS:
+        if not rocks.PATH_MAPPING.is_file():
+            retrieve_mappings()
+
+        with open(rocks.PATH_MAPPING, "r") as file_:
+            rocks.MAPPINGS = json.load(file_)
+    return rocks.MAPPINGS
+
+
 # ------
 # Numerical methods
 def weighted_average(catalogue, parameter):
@@ -90,11 +97,16 @@ def weighted_average(catalogue, parameter):
     float
         The standard error of the weighted average.
     """
+    catalogue = catalogue[catalogue[parameter] != 0]
 
     values = catalogue[parameter]
 
     if parameter in ["albedo", "diameter"]:
         preferred = catalogue[f"preferred_{parameter}"]
+
+        if not any(preferred):
+            preferred = ~preferred
+
         errors = catalogue[f"err_{parameter}_up"]
     else:
         preferred = catalogue["preferred"]
@@ -144,21 +156,40 @@ def weighted_average(catalogue, parameter):
 
 # ------
 # Misc
-def retrieve_mappings_from_ssodnet():
+def retrieve_mappings():
     """Retrieve the mappings JSON from SsODNet to the cache directory."""
 
     URL = "https://ssp.imcce.fr/webservices/ssodnet/api/ssocard/metadata_aster.json"
 
     response = requests.get(URL)
 
-    if response.ok:
-        metadata = response.json()
+    if not response.ok:
+        warnings.error(f"Retrieving mappings file failed with URL:\n{URL}")
+        return
 
-        with open(rocks.PATH_MAPPING, "w") as file_:
-            json.dump(metadata["mapping"], file_)
+    metadata = response.json()
+    mappings = metadata["display"]
 
-    else:
-        warnings.warn(f"Retrieving mappings file failed with URL:\n{URL}")
+    # Turn from flattened to nested dict
+    for path, value in mappings.copy().items():
+
+        parent = mappings
+        parameter = path.split(".")[-1]
+
+        for key in path.split("."):
+
+            if key in ["datacloud", "selection"]:
+                continue
+            elif key == parameter:
+                parent[key] = value
+            else:
+                parent = parent.setdefault(key, {})
+
+        del mappings[path]
+
+    # Turn into nested dict
+    with open(rocks.PATH_MAPPING, "w") as file_:
+        json.dump(mappings, file_)
 
 
 def cache_inventory():
@@ -417,14 +448,3 @@ def get_citation_from_mpc(name):
     citation = citation.find("br").next_sibling.next_sibling
     citation = citation.split("[")[0].strip().replace("  ", " ")
     return citation
-
-    # contents = [content for content in citation.contents if not content.tag]
-
-    # alias, *_, citation, ref, ref_number = citation.contents
-
-    # alias = alias.strip()
-    # citation = citation.replace("[Ref:", "").replace("  ", " ").strip()
-    # ref_number = ref_number.replace("]", "").strip()
-    # ref = ref.text
-
-    # return alias, citation, " ".join([ref, ref_number])
